{--------------------------------------------------------------------------
 | Copyright (C) 1992,1993,1994,1995,1996,1997 Dallas Semiconductor Corporation.
 | All rights Reserved. Printed in U.S.A.
 | This software is protected by copyright laws of
 | the United States and of foreign countries.
 | This material may also be protected by patent laws of the United States
 | and of foreign countries.
 | This software is furnished under a license agreement and/or a
 | nondisclosure agreement and may only be used or copied in accordance
 | with the terms of those agreements.
 | The mere transfer of this software does not imply any licenses
 | of trade secrets, proprietary technology, copyrights, patents,
 | trademarks, maskwork rights, or any other form of intellectual
 | property whatsoever. Dallas Semiconductor retains all ownership rights.
 |--------------------------------------------------------------------------
}
unit GenGlob;

interface

uses
   Graphics,iBTMEXPW,WinProcs,Dialogs,SysUtils;

Var
   MainMode : SmallInt;                { mode }
   PortMode : SmallInt;                { port find mode }
   AppName : Array[0..50] of Char;     { application name }
   PAppName : String;                  { Pascal Appname }
   PortNum, PortType : SmallInt;

Const
   { main demo modes }
   MD_CHECKPORT = 0;
   MD_FINDPARTS = 1;
   { port modes }
   P_SHOW_GET_PORT = 1;
   P_GET_PORT = 2;
   P_GET_SESSION = 3;
   P_DO_SETUP = 4;
   P_ERROR_MSG = 10;
   PERSISTENCE = 5;
   { viewers }
   VIEW_NONE = 0;
   VIEW_TEMP = 1;
   VIEW_FILE = 2;
   VIEW_DEBUG = 3;
   VIEW_SWITCH = 4;
   VIEW_DS1991 = 5;
   VIEW_TIME = 6;
   VIEW_UDP = 7;
   PRIMARY = 0;
   SECONDARY = 1;
   TERTIARY = 2;
   { DEBUG modes }
   DB_IDLE = 0;
   DB_FINDSOURCE = 1;
   DB_READ = 2;
   DB_WRITE = 3;
   DB_VERIFY = 4;
   DB_CHECKPORT = 5;
   DB_FINDDEST = 6;
   DB_WRITENV = 7;
   DB_WRITEEPROM = 8;
   DB_EMPTY = 9;
   { read sub modes }
   DBS_CREATE = 0;
   DBS_READ = 1;
   DBS_DISPLAY = 2;
   { return codes }
   DBIN_PROGRESS = 2;
   DBDONE = 4;
   DBSUCCESS = 1;
   DBNO_DEVICE = -1;
   DBBAD_WRITE = -2;
   DBNON_EMPTY = -3;
   DB_REDIRECT = -4;
   DBNO_SESSION = -200;
   { Colors }
   CLR_LENGTH = 0;
   CLR_PRINT = 1;
   CLR_NONPRINT = 2;
   CLR_CRC = 3;
   CLR_EMPTY = 4;
   { page type }
   SCRATCH = 0;
   MEMORY = 1;
   STATUS = 2;
   APPL = 3;
   NONE = 4;
   { memory type }
   MEM_NONE = 0;
   MEM_NV = 1;
   MEM_EPROM = 2;
   MEM_EE = 3;
   MEM_MNV = 4;
   MEM_SECURE = 5;
   { Window States }
   WNORMAL = 0;
   WMAX = 1;


Function HexCharToByte(ch : Char): Byte;
Function WriteScratch(Addr : SmallInt; Buf : PChar; Len : SmallInt; MemType : SmallInt): SmallInt;
Function CopyScratch(Addr : SmallInt; Len : SmallInt; MemType : SmallInt): SmallInt;

implementation

{----------------------------------------------------------------------------}
{ Take a hex character and return the binary represetation
}
Function HexCharToByte(ch : Char): Byte;
Begin
   If ((Ord(ch) >= Ord('0')) And (Ord(ch) <= Ord('9'))) Then
      HexCharToByte := Byte(Ord(ch) - $30)
   Else If ((Ord(ch) >= Ord('A')) And (Ord(ch) <= Ord('F'))) Then
      HexCharToByte := Byte(Ord(ch) - $37)
   Else If ((Ord(ch) >= Ord('a')) And (Ord(ch) <= Ord('f'))) Then
      HexCharToByte := Byte(Ord(ch) - $57)
   Else
      HexCharToByte := $FF;
End;

{----------------------------------------------------------------------------}
{ Write and verify the scratchpad of a NVRAM device.  Requires session
  already open.
   Returns:   DBSUCCESS    Success,
              DBNO_DEVICE  Device not there
              DBNO_SESSION Could not get a session
              DBBAD_WRITE  Could not write the scratch
}
Function WriteScratch(Addr : SmallInt; Buf : PChar; Len : SmallInt; MemType : SmallInt): SmallInt;
Var
   flag,rt,i : SmallInt;
Begin
   { set return to known state }
   rt := DBSUCCESS;

   { access and verify it is there }
   flag := TMStrongAccess(SHandle,@StateBuf);
   If (flag = 1) Then
   Begin
      { write scratch command }
      TMTouchByte(SHandle,$0F);
      { address 1 }
      TMTouchByte(SHandle,Addr And $FF);
      { If memory type is not EEPROM }
      If (MemType <> MEM_EE) Then
      Begin
         { address 2 }
         TMTouchByte(SHandle,(Addr shr 8) And $FF);
      End;
      { loop to write the data }
      For i := 0 to (Len - 1) Do
      Begin
         If (TMTouchByte(SHandle,SmallInt(Buf[i])) < 0) Then
            Break;
      End;
      { access and verify it is there }
      flag := TMAccess(SHandle,@StateBuf);
      If (flag = 1) Then
      Begin
         { read scratch command }
         TMTouchByte(SHandle,$AA);
         { If memory type is not EEPROM }
         If (MemType <> MEM_EE) Then
         Begin
            { read address 1 }
            If (TMTouchByte(SHandle,$FF) <> (Addr And $FF)) Then
               rt := DBNO_DEVICE;
            { read address 2 }
            If (TMTouchByte(SHandle,$FF) <> ((Addr shr 8) And $FF)) Then
               rt := DBNO_DEVICE;
            { read offset }
            If (TMTouchByte(SHandle,$FF) <> ((Addr - 1 + Len) Mod 32)) Then
               rt := DBNO_DEVICE;
         End
         { EEPROM }
         Else
         Begin
            { read address 1 }
            If (TMTouchByte(SHandle,(Addr And $FF)) <> (Addr And $FF)) Then
               rt := DBNO_DEVICE;
         End;
         { read back the data and match against buffer }
         For i := 0 to (Len - 1) Do
         Begin
            If (TMTouchByte(SHandle,$FF) <> SmallInt(Buf[i])) Then
            Begin
               rt := DBBAD_WRITE;
               Break;
            End;
         End;
      End
      Else
         rt := DBNO_DEVICE;
   End
   Else
      rt := DBNO_DEVICE;

   { set return value }
   WriteScratch := rt;
End;


{----------------------------------------------------------------------------}
{ Copy a previously written scratch page to final memory.  Requires a
  session already open.

   Returns:   DBSUCCESS    Success,
              DBNO_DEVICE  Device not there
              DBNO_SESSION Could not get a session
}
Function CopyScratch(Addr : SmallInt; Len : SmallInt; MemType : SmallInt): SmallInt;
Var
   flag,rt,i,tg : SmallInt;
   M : LongInt;
Begin
   { set return to known state }
   rt := DBSUCCESS;
   flag := 0;

   { access and verify it is there }
   flag := TMStrongAccess(SHandle,@StateBuf);
   If (flag = 1) Then
   Begin
      { copy scratch command }
      TMTouchByte(SHandle,$55);
      { If memory type is not EEPROM }
      If (MemType <> MEM_EE) Then
      Begin
         { address 1 }
         TMTouchByte(SHandle,Addr And $FF);
         { address 2 }
         TMTouchByte(SHandle,(Addr shr 8) And $FF);
         { send offset }
         TMTouchByte(SHandle,((Addr - 1 + Len) Mod 32));
         { get the result of the copy }
         flag := TMTouchByte(SHandle,$FF);
      End
      { EEPROM }
      Else
      Begin
         { send validation key }
         TMTouchByte(SHandle,$A5);
         { wait for a while 2ms }
         M := GetTickCount + 2;
         While (M > GetTickCount) Do;
      End;

      { check validation }
      { special case of monitary NV ram device toggle validation }
      If (MemType = MEM_MNV) Then
      Begin
         tg := 0;
         If ((flag And $10) = $10) Then
            tg := 1;
         flag := flag shr 5;
         For i := 1 to 3 Do
         Begin
            { if this bit opposite to the next }
            If (((Not tg) And $01) <> (flag And $01)) Then
            Begin
               rt := DBNO_DEVICE;
               break;
            End;
            { read for the next bit }
            tg := flag And $01;
            flag := flag shr 1;
         End;
      End
      { regular validation }
      Else
      Begin
         If ((flag And $F0) <> $00) Then
            rt := DBNO_DEVICE
         Else
            rt := DBSUCCESS;
      End;
   End
   Else
      rt := DBNO_DEVICE;

   { set return value }
   CopyScratch := rt;
End;


end.
