{--------------------------------------------------------------------------
 | Copyright (C) 1992,1993,1994,1995 Dallas Semiconductor Corporation.
 | All rights Reserved. Printed in U.S.A.
 | This software is protected by copyright laws of
 | the United States and of foreign countries.
 | This material may also be protected by patent laws of the United States
 | and of foreign countries.
 | This software is furnished under a license agreement and/or a
 | nondisclosure agreement and may only be used or copied in accordance
 | with the terms of those agreements.
 | The mere transfer of this software does not imply any licenses
 | of trade secrets, proprietary technology, copyrights, patents,
 | trademarks, maskwork rights, or any other form of intellectual
 | property whatsoever. Dallas Semiconductor retains all ownership rights.
 |--------------------------------------------------------------------------

   Port selection unit.
}
unit Portun;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Grids, ExtCtrls, StdCtrls, Buttons, GenGlob, IBTMEXPW,
  Tabs, Spin;

type
  TPortForm = class(TForm)
    GridPanel: TPanel;
    CancelPanel: TPanel;
    ClickLabel: TLabel;
    CenterPanel: TPanel;
    DeviceTab: TTabSet;
    SelectBox: TGroupBox;
    PortTypeLabel: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    AdapterLabel: TLabel;
    Label3: TLabel;
    PortNumSpin: TSpinEdit;
    AdvanceGroup: TGroupBox;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    VersionLabel: TLabel;
    DateLabel: TLabel;
    CancelButton: TButton;
    OkButton: TButton;
    MiscMemo: TMemo;
    MessureLabel: TLabel;
    procedure FormCreate(Sender: TObject);
    Function getToken(verStr : String; token : SmallInt): String;
    procedure CancelButtonClick(Sender: TObject);
    procedure DeviceTabChange(Sender: TObject; NewTab: Integer;
      var AllowChange: Boolean);
    Function  SelectPort(NPortNum, NPortType : SmallInt): Boolean;
    procedure OkButtonClick(Sender: TObject);
  private
    PortCnt: SmallInt;
    SessionTimeout: SmallInt;
    ErrorMsg: String;
    JustStarting : Boolean;
    PortTypeValues: Array[0..15] of SmallInt;
    EatEvent : Boolean;
    GotCancel : Boolean;
    { Private declarations }
  public
    { Public declarations }
    PortName : String;
    TPortNum,TPortType: SmallInt;
    Function PortIdle: Boolean;
  end;

var
  PortForm: TPortForm;

implementation

Uses
   IBGENUN;

Const
   TOKEN_ABRV = 0;
   TOKEN_DEV  = 1;
   TOKEN_VER  = 2;
   TOKEN_DATE = 3;
   TOKEN_TAIL = 255;


{$R *.DFM}

{----------------------------------------------------------------------------}
{ Setup the known types in the port/type grid
}
procedure TPortForm.FormCreate(Sender: TObject);
Var
   x,y,len,cnt: SmallInt;
   buf : array[0..200] of Char;
begin
   { dynamically label the available drivers }
   cnt := 0;
   For x := 0 to 15 Do
   Begin
      If (TMGetTypeVersion(x,@buf) > 0) Then
      Begin
         DeviceTab.Tabs.Add(' ' + getToken(StrPas(buf),TOKEN_DEV) + ' ');
         PortTypeValues[cnt] := x;
         Inc(cnt);
      End;
   End;

   { check for no drivers }
   If (cnt = 0) Then
   Begin
      MessageBeep(MB_OK);
      Application.MessageBox('Error, no iButton-TMEX drivers found!',AppName,MB_APPLMODAL Or MB_OK Or MB_ICONSTOP);
      Application.Terminate;
   End
   Else
   Begin
      { check if port type done yet }
      If ((PortNum = -1) Or (PortType = -1)) Then
      Begin
         TPortNum := 1;
         TPortType := 1;
      End
      Else
      Begin
         TPortNum := PortNum;
         TPortType := PortType;
      End;

      { initialized flag }
      EatEvent := False;
      GotCancel := False;
      PortName := '';

      JustStarting := True;
   End;
End;

{----------------------------------------------------------------------------}
{ Select the port
}
Function TPortForm.SelectPort(NPortNum, NPortType : SmallInt): Boolean;
Var
   tab : SmallInt;
   rt : Boolean;
   buf : array[0..200] of Char;
   infoStr,abrStr : String;
Begin
   rt := False;
   For tab := 0 to 15 Do
   Begin
      If (PortTypeValues[tab] = NPortType) Then
      Begin
         { select that tab (if not already selected) }
         If (DeviceTab.TabIndex <> tab) Then
         Begin
            { eat the change tab event }
            EatEvent := True;
            { make sure that this tab is available V310B8 }
            If (tab < DeviceTab.Tabs.Count) Then
               DeviceTab.TabIndex := tab
            Else
            Begin
               { something very bad V310B8 }
               TPortNum := 1;
               TPortType := PortTypeValues[0];
               rt := False;
               EatEvent := False;
               Break;
            End;
            EatEvent := False;
         End;

         If (TMGetTypeVersion(NPortType,@buf) > 0) Then
         Begin
            infoStr := StrPas(buf);
            { get the abreviation }
            abrStr := getToken(infoStr,TOKEN_ABRV);
            { translate to a PC port if possible }
            If (Pos('COM',UpperCase(infoStr)) > 0) Then
               abrStr := 'COM'
            Else If (Pos('LPT',UpperCase(infoStr)) > 0) Then
               abrStr := 'LPT';
            PortTypeLabel.Caption := abrStr;
            { adapter type }
            AdapterLabel.Caption := getToken(infoStr,TOKEN_DEV);
            { Version }
            VersionLabel.Caption := getToken(infoStr,TOKEN_VER);
            { Date }
            DateLabel.Caption := getToken(infoStr,TOKEN_DATE);
            { misc }
            MessureLabel.Caption := getToken(infoStr,TOKEN_TAIL);
            If (MessureLabel.Width < (AdvanceGroup.ClientWidth - MiscMemo.Left)) Then
               MiscMemo.ScrollBars := ssNone
            Else
               MiscMemo.ScrollBars := ssHorizontal;
            MiscMemo.Text := MessureLabel.Caption;

            { number }
            PortNumSpin.Value := NPortNum;
            { success }
            rt := True;
            Break;
         End;
      End;
   End;

   SelectPort := rt;
End;

{----------------------------------------------------------------------------}
{ Parse the version string for a token
}
Function TPortForm.getToken(verStr : String; token : SmallInt): String;
Var
   currentToken : SmallInt;
   i : SmallInt;
   rtStr : String;
   inToken : Boolean;
Begin
   rtStr := '';
   currentToken := -1;
   inToken := False;
   For i := 1 to length(verStr) Do
   Begin
      If ((verStr[i] <> ' ') And (Not inToken)) Then
      Begin
         inToken := True;
         Inc(currentToken);
      End;

      If ((verStr[i] = ' ') And (inToken)) Then
         inToken := False;

      If (((inToken) And (currentToken = token)) Or
          ((token = 255) And (currentToken > TOKEN_DATE))) Then
         rtStr := rtStr + verStr[i];
   End;

   getToken := rtStr;
End;

{----------------------------------------------------------------------------}
{ Idle action for port search
}
Function TPortForm.PortIdle: Boolean;
Var
   i,flag,cnt: SmallInt;
   ans: SmallInt;
   T,M: Longint;
   tbuf: Array[0..257] of Char;
Begin
   PortIdle := False;
   { show the get port form }
   If (PortMode = P_SHOW_GET_PORT) Then
   Begin
      SelectPort(TPortNum,TPortType);
      If (Not PortForm.Visible) Then
         PortForm.ShowModal;
      PortMode := P_GET_PORT;
   End
   { wait for a port to be chosen }
   Else If (PortMode = P_GET_PORT) Then
   Begin
      { check for cancel }
      If (GotCancel) Then
      Begin
         GotCancel := False;
         PortIdle := True;
      End
      Else If (Not PortForm.Visible) Then
      Begin
         PortMode := P_GET_SESSION;
         PortCnt := 0;
         { get tick limit (equivelant to 3 seconds) }
         SessionTimeout := 10;
      End
   End
   { get a session to the new port }
   Else If (PortMode = P_GET_SESSION) Then
   Begin
      { create the port name }
      If (SelectPort(TPortNum,TPortType)) Then
         PortName := ' ' +  AdapterLabel.Caption + ' <' + PortTypeLabel.Caption
                + IntToStr(TPortNum) + '>'
      Else
      Begin
         { something bad happened, the selected port does not have a tab V310B8}
         PortMode := P_SHOW_GET_PORT;
         TPortNum := 1;
         TPortType := PortTypeValues[0];
         Exit;
      End;

      { attempt to start a session on port }
      SHandle := TMExtendedStartSession(TPortNum,TPortType,nil);
      { session successfull }
      If (SHandle > 0) Then
      Begin
         PortMode := P_DO_SETUP;
         PortCnt := 0;
      End
      Else
         Inc(PortCnt);

      { check for too many tries }
      If (PortCnt > SessionTimeout) Then
      Begin
         { error message  }
         ErrorMsg := 'Could not get access to the selected Port: ' +
                     '    '+ PortName +
                     Chr($0A) + Chr($0D) +  Chr($0A) + Chr($0D) +
                     'Another TMEX application has control over this port' + Chr($0A) + Chr($0D) +
                     'or the port type is invalid.' + Chr($0A) + Chr($0D) +
                      Chr($0A) + Chr($0D) + 'RETRY to select another?';
         PortMode := P_ERROR_MSG;
      End;
   End
   { do a tmsetup on the new port }
   Else If (PortMode = P_DO_SETUP) Then
   Begin
      { do the setup }
      flag := TMSetup(SHandle);
      { for Mike Bolan's Laptop }
      if (flag = 0) Then
      Begin
         For i := 0 to 2 do
            flag := TMSetup(SHandle);
      End;

      { check for short condition (slow to power up brick) }
      If ((flag = 1) Or (flag = 2)) Then
      Begin
         cnt := 0;
         For i := 0 to 20 do
         Begin
            { time the setup }
            T := GetTickCount;
            ans := TMTouchReset(SHandle);
            { check for verrrry lonnnng reset }
            If ((GetTickCount - T) > 180) Then   { (3.10) }
               ans := 3;

            If ((ans < 3) And (ans >= 0)) Then
               Inc(cnt);
         End;

         If (cnt > 10) Then
            flag := 1
         Else
            flag := 2;
      End;
      { end the session }
      TMEndSession(SHandle);

      { check results of setup }
      If (flag = 1) Then
      Begin
         { success }
         PortNum := TPortNum;
         PortType := TPortType;
         JustStarting := False;
         PortIdle := True;
      End
      Else
      Begin
         { error message }
         ErrorMsg := 'Could not verify the MicroLAN Port: ' +
                     '    '+ PortName +
                     Chr($0A) + Chr($0D) +  Chr($0A) + Chr($0D) +
                     'Is this a valid port and is the correct' + Chr($0A) + Chr($0D) +
                     'hardware (adapter) connected?' + Chr($0A) + Chr($0D) +
                     Chr($0A) + Chr($0D) + 'RETRY to select another?';
         PortMode := P_ERROR_MSG;
      End;
   End
   { get a session to the new port }
   Else If (PortMode = P_ERROR_MSG) Then
   Begin
      { turn off the idle timer }
      iBGenForm.IdleTimer.Enabled := False;
      { display error and ask for a retry }
      MessageBeep(MB_ICONHAND);
      ans := Application.MessageBox(StrPCopy(tbuf,ErrorMsg),AppName,MB_APPLMODAL Or MB_RETRYCANCEL Or MB_ICONSTOP);
      If (ans = ID_CANCEL) Then
      Begin
         { check if have a port/type to fall back on }
         If ((PortNum = -1) Or (PortType = -1) Or JustStarting) Then
            Application.Terminate
         Else
         Begin
            PortIdle := True;
            { end port search, keep the old ones }
            SelectPort(PortNum,PortType);
            PortName := ' ' +  AdapterLabel.Caption + ' <' + PortTypeLabel.Caption
                + IntToStr(PortNum) + '>';
         End;
      End
      Else
         PortMode := P_SHOW_GET_PORT;
      { turn back on the idle timer }
      iBGenForm.IdleTimer.Enabled := True;
   End;
End;


{----------------------------------------------------------------------------}
{ Failed to find a Port/Type so end the program
}
procedure TPortForm.CancelButtonClick(Sender: TObject);
begin
   { check if have a port/type to fall back on }
   If ((PortNum = -1) Or (PortType = -1) Or JustStarting) Then
      Application.Terminate
   Else
   Begin
      GotCancel := True;
      ModalResult := idOK;
      PortForm.Hide;
   End;
end;

{----------------------------------------------------------------------------}
{ Select a diffenent Port type
}
procedure TPortForm.DeviceTabChange(Sender: TObject; NewTab: Integer;
  var AllowChange: Boolean);
begin
   If (Not EatEvent) Then
      SelectPort(1,PortTypeValues[NewTab]);
end;

{----------------------------------------------------------------------------}
{ Finished selection event
}
procedure TPortForm.OkButtonClick(Sender: TObject);
begin
   TPortNum := PortNumSpin.Value;
   TPortType := PortTypeValues[DeviceTab.TabIndex];
   ModalResult := idOK;
   PortForm.Hide;
end;

end.
